part of sliders;

mixin _SliderMixin {
  Path path = Path();

  double minTooltipWidth = 47.0;

  double minTooltipHeight = 37.0;

  double halfMinTooltipWidth = 23.5;

  double cornerRadius = 4.0;

  /// Get the minimum and maximum value based on the
  /// intervalType to find the exact range.
  int _getDateTimeDifference(
      dynamic min, dynamic max, DateIntervalType intervalType) {
    final Duration diff = max.difference(min);
    switch (intervalType) {
      case DateIntervalType.months:
        return ((max.year - min.year) * DateTime.monthsPerYear) +
            max.month -
            min.month;
        break;
      case DateIntervalType.days:
        return diff.inDays;
        break;
      case DateIntervalType.hours:
        return diff.inHours;
        break;
      case DateIntervalType.minutes:
        return diff.inMinutes;
        break;
      case DateIntervalType.seconds:
        return diff.inSeconds;
        break;
      default:
        return max.year - min.year;
        break;
    }
  }

  /// Get the date time label based on the interval and intervalType.
  DateTime _getNextDate(
      DateTime currentDate, DateIntervalType intervalType, double interval) {
    switch (intervalType) {
      case DateIntervalType.months:
        // Make the label start date will always be 1 other than first label.
        return DateTime(
            currentDate.year, currentDate.month + interval.ceil(), 1);
        break;
      case DateIntervalType.days:
        currentDate = currentDate.add(Duration(days: interval.ceil()));
        return DateTime(currentDate.year, currentDate.month, currentDate.day);
        break;
      case DateIntervalType.hours:
        currentDate = currentDate.add(Duration(hours: interval.ceil()));
        return DateTime(currentDate.year, currentDate.month, currentDate.day,
            currentDate.hour);
        break;
      case DateIntervalType.minutes:
        return currentDate.add(Duration(minutes: interval.ceil()));
        break;
      case DateIntervalType.seconds:
        return currentDate.add(Duration(seconds: interval.ceil()));
        break;
      default:
        return DateTime(currentDate.year + interval.ceil(), 1, 1);
        break;
    }
  }

  void _drawText(
      PaintingContext context, Offset center, Offset thumbCenter, String text,
      {Offset endThumbCenter,
      RenderProxyBox parentBox,
      SfRangeSliderThemeData themeData,
      Animation<double> animation,
      TextPainter textPainter,
      TextDirection textDirection}) {
    bool isInactive;
    switch (textDirection) {
      case TextDirection.ltr:
        isInactive =
            center.dx < thumbCenter.dx || center.dx > endThumbCenter.dx;
        break;
      case TextDirection.rtl:
        isInactive =
            center.dx > thumbCenter.dx || center.dx < endThumbCenter.dx;
        break;
    }

    final TextSpan textSpan = TextSpan(
      text: text,
      style: isInactive
          ? themeData.inactiveLabelStyle
          : themeData.activeLabelStyle,
    );
    textPainter.text = textSpan;
    textPainter.layout();
    textPainter.paint(
        context.canvas, Offset(center.dx - textPainter.width / 2, center.dy));
  }

  Path getTooltipPath(Size textSize, double tooltipStartY,
      Offset trackStartOffset, Rect trackRect, double dx, Size parentSize) {
    final double dy = tooltipStartY + _tooltipTriangleHeight;
    final double tooltipWidth =
        textSize.width < minTooltipWidth ? minTooltipWidth : textSize.width;
    final double tooltipHeight =
        textSize.height < minTooltipHeight ? minTooltipHeight : textSize.height;
    final double halfTooltipWidth = tooltipWidth / 2;

    double rightLineWidth =
        dx + halfTooltipWidth > trackRect.width + trackStartOffset.dx
            ? trackRect.width + trackStartOffset.dx - dx
            : halfTooltipWidth;
    final double leftLineWidth = dx - halfTooltipWidth < trackRect.left
        ? dx - trackRect.left
        : tooltipWidth - rightLineWidth;
    rightLineWidth = leftLineWidth < halfTooltipWidth
        ? halfTooltipWidth - leftLineWidth + rightLineWidth
        : rightLineWidth;
    const double halfTooltipTriangleWidth = _tooltipTriangleWidth / 2;

    path.reset();
    path.moveTo(dx, trackStartOffset.dy - tooltipStartY);
    //    /
    final bool canAdjustTooltipNose =
        rightLineWidth > halfTooltipTriangleWidth + cornerRadius / 2;
    path.lineTo(
        dx + (canAdjustTooltipNose ? halfTooltipTriangleWidth : rightLineWidth),
        trackStartOffset.dy -
            dy -
            (canAdjustTooltipNose ? 0 : cornerRadius / 2));
    //      ___
    //     /
    path.lineTo(dx + rightLineWidth - cornerRadius, trackStartOffset.dy - dy);
    //      ___|
    //     /
    path.quadraticBezierTo(dx + rightLineWidth, trackStartOffset.dy - dy,
        dx + rightLineWidth, trackStartOffset.dy - dy - cornerRadius);
    path.lineTo(dx + rightLineWidth,
        trackStartOffset.dy - dy - tooltipHeight + cornerRadius);
    //  _______
    //      ___|
    //     /
    path.quadraticBezierTo(
        dx + rightLineWidth,
        trackStartOffset.dy - dy - tooltipHeight,
        dx + rightLineWidth - cornerRadius,
        trackStartOffset.dy - dy - tooltipHeight);
    path.lineTo(dx - leftLineWidth + cornerRadius,
        trackStartOffset.dy - dy - tooltipHeight);
    //  _______
    // |    ___|
    //     /
    path.quadraticBezierTo(
        dx - leftLineWidth,
        trackStartOffset.dy - dy - tooltipHeight,
        dx - leftLineWidth,
        trackStartOffset.dy - dy - tooltipHeight + cornerRadius);
    path.lineTo(dx - leftLineWidth, trackStartOffset.dy - dy - cornerRadius);
    //  ________
    // |___  ___|
    //      /
    if (leftLineWidth > halfTooltipTriangleWidth) {
      path.quadraticBezierTo(dx - leftLineWidth, trackStartOffset.dy - dy,
          dx - leftLineWidth + cornerRadius, trackStartOffset.dy - dy);
      path.lineTo(dx - halfTooltipTriangleWidth, trackStartOffset.dy - dy);
    }
    //  ________
    // |___  ___|
    //     \/
    path.close();
    return path;
  }
}
